@@ -14,22 +14,24 @@ from sympy.logic import simplify_logic
 from sympy import symbols
 from cvehound.cpu import CPU
 from cvehound.exception import UnsupportedVersion
-from cvehound.util import get_spatch_version, get_all_cves, get_cves_metadata
+from cvehound.util import get_spatch_version, get_rule_cves, get_cves_metadata
 from cvehound.kbuild import KbuildParser
 from cvehound.config import Config
 
-__VERSION__ = '1.0.3'
+__VERSION__ = '1.0.7'
 
 
 class CVEhound:
 
-    def __init__(self, kernel, config=None, arch='x86'):
+    def __init__(self, kernel, config=None, check_strict=False, arch='x86'):
+        kernel = os.path.abspath(kernel)
         self.kernel = kernel
         self.metadata = get_cves_metadata()
         self.cocci_job = str(CPU().get_cocci_jobs())
         self.spatch_version = get_spatch_version()
+        self.check_strict = check_strict
         self.rules_metadata = {}
-        self.cve_rules = get_all_cves()
+        (self.cve_all_rules, self.cve_assigned_rules, self.cve_disputed_rules) = get_rule_cves()
 
         ipaths = [
             os.path.join('arch', arch, 'include'),
@@ -69,27 +71,57 @@ class CVEhound:
             self.config = None
 
     def get_grep_pattern(self, rule):
-        is_fix = False
-        start = False
         patterns = []
         with open(rule, 'rt') as fh:
             for line in fh:
                 line = line.strip()
-                if line == 'FIX':
-                    is_fix = True
-                    start = True
+                if not line:
                     continue
-                elif line == 'ERROR':
-                    start = True
+                if line.startswith('//'):
                     continue
-                if start and line:
+                if line:
                     patterns.append(line)
-        return (is_fix, patterns)
+        return patterns
+
+    def _print_found_cve(self, cve):
+        logging.warning('Found: ' + cve)
+        if cve in self.metadata:
+            info = self.metadata[cve]
+            if 'cmt_msg' in info:
+                logging.info('MSG: ' + info['cmt_msg'])
+            if 'cwe' in info:
+                logging.info('CWE: ' + info['cwe'])
+            if 'cvss2' in info and 'score' in info['cvss2']:
+                logging.info('CVSS2: ' + str(info['cvss2']['score']))
+            if 'cvss3' in info and 'score' in info['cvss3']:
+                logging.info('CVSS3: ' + str(info['cvss3']['score']))
+            if 'fix_date' in info:
+                logging.info('FIX DATE: ' + str(datetime.utcfromtimestamp(info['fix_date']).strftime('%Y-%m-%d')))
+        logging.info('https://www.linuxkernelcves.com/cves/' + cve)
+
+    def _print_affected_files(self, config):
+        if config['files']:
+            logging.info('Affected Files:')
+            for file in config['files']:
+                logic = config['files'][file]['logic']
+                if self.config:
+                    affected = 'affected' if config['files'][file]['config'] else 'not affected'
+                    logging.info(' - ' + file + ': ' + logic + '\n   ' + self.config_file + ': ' + affected)
+                else:
+                    logging.info(' - ' + file + ': ' + logic)
+
+        if 'affected' not in config or config['affected'] == None:
+            return
+        config_affected = 'affected' if config['affected'] else 'not affected'
+        if self.config:
+            logging.info('Config: ' + self.config_file + ' ' + config_affected)
+        else:
+            logging.info('Config: any ' + config_affected)
 
     def check_cve(self, cve, all_files=False):
         result = {}
         is_grep = False
-        rule = self.cve_rules[cve]
+        rule = self.cve_all_rules[cve]
         if rule.endswith('.grep'):
             is_grep = True
 
@@ -133,94 +165,76 @@ class CVEhound:
                 if ('Sys_error("' + cve + ': No such file or directory")') not in err:
                     raise e
         else:
-            (is_fix, patterns) = self.get_grep_pattern(rule)
-            args = ['grep', '--include=*.[ch]', '-rPzoe', patterns[0], *files]
-            patterns.pop(0)
-            run = subprocess.run(args, stdout=PIPE, stderr=PIPE, check=False)
-            if run.returncode == 0:
-                output = run.stdout
-                last = patterns.pop()
-                for pattern in patterns:
-                    run = subprocess.run(['grep', '-Pzoe', pattern],
-                                         input=output, check=False,
-                                         stdout=PIPE, stderr=PIPE)
-                    if run.returncode != 0:
-                        output = ''
-                        break
-                    output = run.stdout
-                if run.returncode == 0:
-                    run = subprocess.run(['grep', '-Pzoe', last],
-                                         input=output, check=False,
-                                         stdout=PIPE, stderr=PIPE)
-                    success = run.returncode == 0
-                    if is_fix == success:
-                        output = ''
-                    else:
-                        output = 'ERROR'
+            for pattern in self.get_grep_pattern(rule):
+                args = ['grep', '-rPzle', pattern, *files]
+                run = subprocess.run(args, stdout=PIPE, stderr=PIPE, check=False)
+                if run.returncode != 0:
+                    break
+                output += run.stdout.decode('utf-8').strip()
+            else:
+                # Found all patterns
+                output += '\nERROR'
 
-        if 'ERROR' in output:
-            logging.warning('Found: ' + cve)
-            if cve in self.metadata:
-                info = self.metadata[cve]
-                result = info
-                if 'cmt_msg' in info:
-                    logging.info('MSG: ' + info['cmt_msg'])
-                if 'cwe' in info:
-                    logging.info('CWE: ' + info['cwe'])
-                if 'cvss2' in info and 'score' in info['cvss2']:
-                    logging.info('CVSS2: ' + str(info['cvss2']['score']))
-                if 'cvss3' in info and 'score' in info['cvss3']:
-                    logging.info('CVSS3: ' + str(info['cvss3']['score']))
-                if 'fix_date' in info:
-                    logging.info('FIX DATE: ' + str(datetime.utcfromtimestamp(info['fix_date'])))
-            logging.info('https://www.linuxkernelcves.com/cves/' + cve)
-            if self.config_map:
-                result['config'] = {}
+        if 'ERROR' not in output:
+            return False
+
+        if self.config_map:
+            config_result = {}
+            files = {}
+            for line in output.split('\n'):
+                file = []
+                if not is_grep:
+                    file = [ line.split(':')[0] ]
+                else:
+                    while True:
+                        try:
+                            rindex = line.rindex(self.kernel)
+                        except ValueError:
+                            break
+                        file.append(line[rindex:])
+                        line = line[:rindex]
+                for f in filter(lambda f: os.path.isfile(f), file):
+                    files[f] = self.config_map.get(f, '')
+            if files:
                 config_affected = None
-                files = {}
-                for line in output.split('\n'):
-                    file = line.split(':')[0]
-                    files[file] = self.config_map.get(file, '')
-                if files:
-                    logging.info('Affected Files:')
-                    for file, config in files.items():
-                        result['config'][file] = {}
-                        if config:
-                            config = simplify_logic(config)
-                            result['config'][file]['logic'] = str(config)
-                            if self.config:
-                                affected = config.subs(self.config.get_mapping())
-                                if affected == True:
-                                    affected = 'affected'
-                                    result['config'][file]['config'] = True
-                                    config_affected = True
-                                else:
-                                    affected = 'not affected'
-                                    result['config'][file]['config'] = False
-                                    if config_affected == None:
-                                        config_affected = False
-                                logging.info(' - ' + file + ': ' + str(config) + '\n   ' + self.config_file + ': ' + affected)
+                if 'files' not in config_result:
+                    config_result['files'] = {}
+                for file, config in files.items():
+                    rel_file = file[len(self.kernel)+1:]
+                    result_file = {}
+                    if config:
+                        config = simplify_logic(config)
+                        result_file['logic'] = str(config)
+                        if self.config:
+                            affected = config.subs(self.config.get_mapping())
+                            if affected == True:
+                                result_file['config'] = True
+                                config_affected = True
                             else:
-                                logging.info(' - ' + file + ': ' + str(config))
-                        elif not file.endswith('.h'): # TODO: if only .h file, e.g. linux/kernel.h?
-                            result['config'][file]['logic'] = True
-                            result['config'][file]['config'] = True
-                            config_affected = True
-                            logging.info(' - ' + file + ': True')
-                result['config']['affected'] = config_affected
-                if config_affected != None:
-                    affected = 'affected'
-                    if not config_affected:
-                        affected = 'not affected'
-                    if self.config:
-                        logging.info('Config: ' + self.config_file + ' ' + affected)
-                    else:
-                        logging.info('Config: any ' + affected)
+                                result_file['config'] = False
+                                if config_affected == None:
+                                    config_affected = False
+                    elif not file.endswith('.h'): # TODO: if only .h file, e.g. linux/kernel.h?
+                        result_file['logic'] = True
+                        result_file['config'] = True
+                        config_affected = True
+                    config_result['files'][rel_file] = result_file
+            if config_affected != None:
+                config_result['affected'] = config_affected
+
+        if self.check_strict and 'affected' in config_result and config_result['affected'] or not self.check_strict:
+            if cve in self.metadata:
+                result = self.metadata[cve]
+            result['config'] = config_result
             result['spatch_output'] = output
+            self._print_found_cve(cve)
+            self._print_affected_files(config_result)
             logging.debug(output)
             logging.info('')
-            return result
-        return False
+        else:
+            return False
+
+        return result
 
     def get_rule_metadata(self, cve):
         files = []
@@ -231,7 +245,7 @@ class CVEhound:
         if cve in self.rules_metadata:
             return self.rules_metadata[cve]
 
-        with open(self.cve_rules[cve], 'rt') as fh:
+        with open(self.cve_all_rules[cve], 'rt') as fh:
             for line in fh:
                 if not line.startswith('///'):
                     break
@@ -262,11 +276,20 @@ class CVEhound:
     def get_cve_cwe(self, cve):
         return self.metadata[cve].get('cwe', None)
 
-    def get_cves(self):
-        return self.cve_rules.keys()
+    def get_cve_exploit(self, cve):
+        return self.metadata[cve]['exploit']
+
+    def get_all_cves(self):
+        return set(self.cve_all_rules.keys())
+
+    def get_assigned_cves(self):
+        return set(self.cve_assigned_rules.keys())
+
+    def get_disputed_cves(self):
+        return set(self.cve_disputed_rules.keys())
 
     def get_rule(self, cve):
-        return self.cve_rules[cve]
+        return self.cve_all_rules[cve]
 
     def get_rule_fix(self, cve):
         return self.get_rule_metadata(cve)['fix']
